using System;
using System.Collections.Generic;
using System.Windows.Forms;
using System.IO;
using System.Text.RegularExpressions;

namespace LuXr.Plugins.l
{
    class l : LuXr.SelectorPlugin
    {
        Launcher launcher;

        bool more = false;

        public l()
        {
            setMetaInfo("Program Launcher", "Peter Thoman", "0.3");
            settings = new SettingsPanel(new Setting[] {
                new BoolSetting("comProg", "Search common program folder"),
                new BoolSetting("userProg", "Search user-specific program folder"),
                new BoolSetting("win", "Search windows folder for executables"),
                new BoolSetting("winSys", "Search system32 folder for executables") });
        }

        public override void Initialize()
        {
            launcher = new Launcher(settings);
        }

        override public bool Handle(string cmd, bool enter)
        {
            if (cmd.Length < 2)
            {
                messageBox.Text = "Launch: Need at least 2 characters.";
                selectedIndex = -1;
                return false;
            }
            messageBox.Text = "Launch:\n";
            List<Launcher.Item> items = launcher.MatchingItems(cmd);
            if (!enter) selectedIndex = items.Count > 0 ? 0 : -1;
            int c = 0;
            more = false;
            foreach (Launcher.Item i in items)
            {
                if (c++ > 6) { messageBox.Text += "..."; more = true; break; }
                messageBox.Text += i.Name + "\n";
            }
            if (enter)
            {
                if (items.Count > 0)
                {
                    List<Launcher.Item>.Enumerator e = items.GetRange(selectedIndex, 1).GetEnumerator();
                    e.MoveNext();
                    e.Current.Launch();
                    return true;
                }
            }
            return false;
        }

        override public void HandleCommandKey(Keys keyData)
        {
            base.HandleCommandKey(keyData);
            if (selectedIndex == 7 && more)
                selectedIndex = 6;
        }
    }

    class Launcher
    {
        public class Item
        {
            string name, launch;
            public string Name { get { return name; } }

            public Item(string name, string launchfile)
            {
                this.name = name;
                launch = launchfile;
            }

            internal void Launch()
            {
                System.Diagnostics.Process.Start(launch);
            }

            public override bool Equals(Object o)
            { return (o.GetType() == GetType()) && name.Equals(((Item)o).name); }
            public override int GetHashCode()
            { return name.GetHashCode(); }
        }

        public class ItemList : List<Item>
        {
            public void SingleAdd(Item e)
            {
                if (!this.Contains(e)) base.Add(e);
            }
        }
        ItemList items = new ItemList();

        const string SHELL_FOLDER_PATH = "\\SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Explorer\\Shell Folders";
        public Launcher(SettingsPanel settings)
        {
            string commonProgramsFolder = (string)Microsoft.Win32.Registry.GetValue("HKEY_LOCAL_MACHINE" + SHELL_FOLDER_PATH, "Common Programs", null);
            string userProgramsFolder = (string)Microsoft.Win32.Registry.GetValue("HKEY_CURRENT_USER" + SHELL_FOLDER_PATH, "Programs", null);
            if (commonProgramsFolder == null || userProgramsFolder == null)
            {
                Util.showError("Could not find common programs folder. Launcher functionality will be limited.");
                return;
            }
            string windowsFolder = Environment.GetEnvironmentVariable("SystemRoot");

            if ((bool)settings["comProg"]) RecursiveAdd(new DirectoryInfo(commonProgramsFolder), "*.lnk");
            if ((bool)settings["userProg"]) RecursiveAdd(new DirectoryInfo(userProgramsFolder), "*.lnk");
            if ((bool)settings["win"]) RecursiveAdd(new DirectoryInfo(windowsFolder), "*.exe");
            if ((bool)settings["winSys"]) RecursiveAdd(new DirectoryInfo(windowsFolder + "\\system32"), "*.exe");
        }

        private void RecursiveAdd(DirectoryInfo d, string mask)
        {
            foreach (FileInfo f in d.GetFiles(mask))
                items.SingleAdd(new Item(f.Name.Remove(f.Name.Length - 4), d.FullName + "\\" + f.Name));
            foreach (DirectoryInfo df in d.GetDirectories())
                RecursiveAdd(df, mask);
        }

        public List<Item> MatchingItems(string search)
        {
            string[] matchStrings = search.Split(new char[] { ' ' });
            List<Regex> matchRegs = new List<Regex>();
            foreach (string s in matchStrings)
                matchRegs.Add(new Regex(Regex.Escape(s), RegexOptions.IgnoreCase));

            List<Item> result = items.FindAll(delegate(Item i)
            {
                return matchRegs.TrueForAll(delegate(Regex r)
                {
                    return r.IsMatch(i.Name);
                });
            });
            result.Sort(delegate(Item i, Item j)
            {
                return i.Name.Length - j.Name.Length;
            });
            return result;
        }
    }
}